Package: MutationalPatterns
Type: Package
Title: Comprehensive genome-wide analysis of mutational processes
Description: Mutational processes leave characteristic footprints in genomic DNA. This
    package provides a comprehensive set of flexible functions that allows
    researchers to easily evaluate and visualize a multitude of mutational patterns
    in base substitution catalogues of e.g. healthy samples, tumour samples, or
    DNA-repair deficient cells. The package covers a wide range of patterns including: mutational
    signatures, transcriptional and replicative strand bias, lesion segregation,
    genomic distribution and association with genomic features, which are
    collectively meaningful for studying the activity of mutational processes. The
    package works with single nucleotide variants (SNVs), insertions and deletions
    (Indels), double base substitutions (DBSs) and larger multi base substitutions
    (MBSs). The package provides functionalities for both extracting mutational
    signatures de novo and determining the contribution of previously identified
    mutational signatures on a single sample level. MutationalPatterns integrates
    with common R genomic analysis workflows and allows easy association with
    (publicly available) annotation data.
Version: 3.22.0
Date: 2025-07-30
Authors@R: c(
    person("Freek", "Manders", email = "F.M.Manders@prinsesmaximacentrum.nl", role = c("aut"), 
    comment = c(ORCID = "0000-0001-6197-347X")),
    person("Francis", "Blokzijl", role = "aut",
    comment = c(ORCID = "0000-0002-8084-8444")),
    person("Roel", "Janssen", role = "aut",
    comment = c(ORCID = "0000-0003-4324-5350")),
    person("Jurrian", "de Kanter", role = "ctb",
    comment = c(ORCID = "0000-0001-5665-3711")),
    person("Rurika", "Oka", role = "ctb",
    comment = c(ORCID = "0000-0003-4107-7250")),
    person("Mark", "van Roosmalen", email = "vanBoxtelBioinformatics@prinsesmaximacentrum.nl", role = "cre"),
    person("Ruben", "van Boxtel", role = c("aut", "cph"),
    comment = c(ORCID = "0000-0003-1285-2836")),
    person("Edwin", "Cuppen", role = "aut",
    comment = c(ORCID = "0000-0002-0400-9542")))
License: MIT + file LICENSE
URL: https://doi.org/doi:10.1186/s12864-022-08357-3
Imports: stats, S4Vectors, BiocGenerics (>= 0.18.0), BSgenome (>=
        1.40.0), VariantAnnotation (>= 1.18.1), dplyr (>= 0.8.3),
        tibble(>= 2.1.3), purrr (>= 0.3.2), tidyr (>= 1.0.0), stringr
        (>= 1.4.0), magrittr (>= 1.5), ggplot2 (>= 2.1.0), pracma (>=
        1.8.8), IRanges (>= 2.6.0), Seqinfo, GenomeInfoDb (>= 1.45.9),
        Biostrings (>= 2.40.0), ggdendro (>= 0.1-20), cowplot (>=
        0.9.2), ggalluvial (>= 0.12.2), RColorBrewer, methods
Depends: R (>= 4.2.0), GenomicRanges (>= 1.24.0), NMF (>= 0.20.6)
Suggests: BSgenome.Hsapiens.UCSC.hg19 (>= 1.4.0), BiocStyle (>= 2.0.3),
        TxDb.Hsapiens.UCSC.hg19.knownGene (>= 3.2.2), biomaRt (>=
        2.28.0), gridExtra (>= 2.2.1), rtracklayer (>= 1.32.2), ccfindR
        (>= 1.6.0), GenomicFeatures, AnnotationDbi, testthat, knitr,
        rmarkdown
biocViews: Genetics, SomaticMutation
ZipData: NA
RoxygenNote: 7.3.2
Encoding: UTF-8
VignetteBuilder: knitr
git_url: https://git.bioconductor.org/packages/MutationalPatterns
git_branch: RELEASE_3_23
git_last_commit: 46b540d
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 23:40:31 UTC; biocbuild
Author: Freek Manders [aut] (ORCID: <https://orcid.org/0000-0001-6197-347X>),
  Francis Blokzijl [aut] (ORCID: <https://orcid.org/0000-0002-8084-8444>),
  Roel Janssen [aut] (ORCID: <https://orcid.org/0000-0003-4324-5350>),
  Jurrian de Kanter [ctb] (ORCID:
    <https://orcid.org/0000-0001-5665-3711>),
  Rurika Oka [ctb] (ORCID: <https://orcid.org/0000-0003-4107-7250>),
  Mark van Roosmalen [cre],
  Ruben van Boxtel [aut, cph] (ORCID:
    <https://orcid.org/0000-0003-1285-2836>),
  Edwin Cuppen [aut] (ORCID: <https://orcid.org/0000-0002-0400-9542>)
Maintainer: Mark van Roosmalen <vanBoxtelBioinformatics@prinsesmaximacentrum.nl>
