# Generated by roxygen2: do not edit by hand

export(bin_mutation_density)
export(binomial_test)
export(calculate_lesion_segregation)
export(cluster_signatures)
export(context_potential_damage_analysis)
export(convert_sigs_to_ref)
export(cos_sim)
export(cos_sim_matrix)
export(count_dbs_contexts)
export(count_indel_contexts)
export(count_mbs_contexts)
export(determine_regional_similarity)
export(enrichment_depletion_test)
export(extract_signatures)
export(fit_to_signatures)
export(fit_to_signatures_bootstrapped)
export(fit_to_signatures_strict)
export(genomic_distribution)
export(get_dbs_context)
export(get_indel_context)
export(get_known_signatures)
export(get_mut_type)
export(get_sim_tb)
export(lengthen_mut_matrix)
export(merge_signatures)
export(mut_context)
export(mut_matrix)
export(mut_matrix_stranded)
export(mut_strand)
export(mut_type)
export(mut_type_occurrences)
export(mutations_from_vcf)
export(plot_192_profile)
export(plot_96_profile)
export(plot_bootstrapped_contribution)
export(plot_compare_dbs)
export(plot_compare_indels)
export(plot_compare_mbs)
export(plot_compare_profiles)
export(plot_contribution)
export(plot_contribution_heatmap)
export(plot_correlation_bootstrap)
export(plot_cosine_heatmap)
export(plot_dbs_contexts)
export(plot_enrichment_depletion)
export(plot_indel_contexts)
export(plot_lesion_segregation)
export(plot_main_dbs_contexts)
export(plot_main_indel_contexts)
export(plot_mbs_contexts)
export(plot_original_vs_reconstructed)
export(plot_profile_heatmap)
export(plot_profile_region)
export(plot_rainfall)
export(plot_regional_similarity)
export(plot_river)
export(plot_signature_strand_bias)
export(plot_spectrum)
export(plot_spectrum_region)
export(plot_strand)
export(plot_strand_bias)
export(pool_mut_mat)
export(read_vcfs_as_granges)
export(rename_nmf_signatures)
export(signature_potential_damage_analysis)
export(split_muts_region)
export(strand_bias_test)
export(strand_occurrences)
export(type_context)
exportClasses(region_cossim)
exportMethods(get_sim_tb)
exportMethods(show)
import(GenomicRanges)
import(NMF)
import(ggalluvial)
import(ggplot2)
importFrom(magrittr,"%>%")
importFrom(methods,setClass)
importFrom(methods,setGeneric)
importFrom(methods,setMethod)
importFrom(pracma,lsqnonneg)
